/****************************************************************************
 * Ce programme fait partie du projet Multiserveur.                         *
 * (C) Nicolas GEORGE, aot 1998.                                           *
 * Vous devriez trouver les termes de sa license pas trs loin.             *
 * Je dcline toute responsabilit quant  l'effet de ce programme.         *
 ****************************************************************************/

/******************************************************
  Fonctions de contrle par le propritaire du serveur
 ******************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <fcntl.h>
#include <stdarg.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "config.h"
#include "module.h"
#include "multiserv.h"

static FILE *inp=NULL,*outp=NULL,*outf;

void initialiser_superviseur(void)
{
  remove(INPIPE);
  mkfifo(INPIPE,INPIPEMODE);
  remove(OUTPIPE);
  mkfifo(OUTPIPE,OUTPIPEMODE);
  outf=fopen(LOGFILE,"a");
  if(outf!=NULL){
    fcntl(fileno(outf),F_SETFL,O_NONBLOCK);
    setbuf(outf,NULL);
  }
  signal(SIGPIPE,SIG_IGN);
}

int try_to_open(char *f,int m,FILE **fd)
{
  int d;

  d=open(f,(m?O_WRONLY:O_RDONLY)|O_NONBLOCK);
  if(d<0)
    return(0);
  *fd=fdopen(d,m?"w":"r");
  setbuf(*fd,NULL);
  return(1);
}

void lprintf(char *f,...)
{
  va_list v;
  time_t tt;
  struct tm t;

  if((outp!=NULL)||try_to_open(OUTPIPE,1,&outp)){
    va_start(v,f);
    if(vfprintf(outp,f,v)<0){
      fclose(outp);
      outp=NULL;
    }
    va_end(v);
  }
  if(outf!=NULL){
    va_start(v,f);
    time(&tt);
    t=*localtime(&tt);
    fprintf(outf,"%2.2d/%2.2d/%4.4d %2.2d:%2.2d:%2.2d : ",t.tm_mday,t.tm_mon+1,t.tm_year+1900,
	    t.tm_hour,t.tm_min,t.tm_sec);
    vfprintf(outf,f,v);
    va_end(v);
  }
}

void pprintf(char *f,...)
{
  va_list v;

  if((outp!=NULL)||try_to_open(OUTPIPE,1,&outp)){
    va_start(v,f);
    if(vfprintf(outp,f,v)<0){
      fclose(outp);
      outp=NULL;
    }
    va_end(v);
  }
}

static int decomposer(char *b,char **a)
{
  int r=0;

  while(1){
    for(;(*b==' ')||(*b=='\t');b++);
    if(*b==0) break;
    *(a++)=b;
    r++;
    if(r==16) break;
    for(;(*b!=' ')&&(*b!='\t')&&(*b!=0);b++);
    if(*b==0) break;
    *(b++)=0;
  }
  return(r);
}

void surveiller_commandes(void)
{
  char b[200],b2[200],*a[16];
  int i,na;

  if((inp==NULL)&&(!try_to_open(INPIPE,0,&inp)))
    return;
  if(fgets(b,199,inp)==NULL){
    if((errno!=EBUSY)&&(errno!=EAGAIN)){
      fclose(inp);
      inp=NULL;
    }
    return;
  }
  b[199]='\n';
  for(i=0;b[i]!='\n';i++);
  b[i]=0;
  strcpy(b2,b);
  na=decomposer(b,a);
  if(na==0)
    return;
  if((strcmp(a[0],"list")==0)||(strcmp(a[0],"ls")==0)||(strcmp(a[0],"l")==0)){
    pprintf("Modules:\n");
    for(i=0;i<nbr_pieces;i++)
      if(pieces[i]!=NULL){
	pieces[i]->class->describe(pieces[i],-1,b2);
	b2[70]=0;
	pprintf("%5d   %s\n",i,b2);
      }
    pprintf("\nClients:\n");
    for(i=0;i<MAXCLI;i++)
      if(client[i].pid!=0)
	pprintf("%5d   (dans %5d)   %s\n",i,clipriv[i].mod,clipriv[i].nom);
  }else if((strcmp(a[0],"echo")==0)&&(na>1)){
    for(i=0;i<MAXCLI;i++)
      if(client[i].pid!=0)
	message_discret(i,b2+(a[1]-b));
  }else if((strcmp(a[0],"echo1")==0)&&(na>2)){
    i=atoi(a[1]);
    if(client[i].pid!=0)
      message_discret(i,b2+(a[2]-b));
    else
      pprintf("Pas de tel client.\n");
  }else if((strcmp(a[0],"welc")==0)&&(na>1)){
    strcpy(ph_bienvenue,b2+(a[1]-b));
    for(i=0;i<MAXCLI;i++)
      if(client[i].pid!=0)
	message_discret(i,b2+(a[1]-b));
  }else if((strcmp(a[0],"vire")==0)&&(na==2)){
    i=atoi(a[1]);
    if(client[i].pid!=0)
      virer_client(i);
    else
      pprintf("Pas de tel client.\n");
  }else if(strcmp(a[0],"eteindre")==0){
    kill(getpid(),SIGTERM);
  }else{
    lprintf("Commande inconnue\n");
  }
}
